package id

import "fmt"

const (
	Revision        = "revision"
	RevisionsPrefix = "revs"
)

type RevisionId struct {
	ItemId
	RevisionID string `json:"rev_id" bson:"rev_id,omitempty"`
}

var _ Descriptor = &RevisionId{}

func (id *RevisionId) New() Descriptor {
	return &RevisionId{}
}

func (id *RevisionId) Type() string { return Revision }

func (id *RevisionId) String() string {
	return Join(id.ItemId.String(), RevisionsPrefix, id.RevisionID)

}

func (id *RevisionId) FromParts(parts []string) error {
	if len(parts) != 10 || parts[8] != RevisionsPrefix {
		return ErrInvalidID
	}
	if err := id.ItemId.FromParts(parts[:8]); err != nil {
		return err
	}
	id.RevisionID = parts[9]
	return nil
}

func (id *RevisionId) Map() map[string]any {
	m := id.ItemId.Map()
	m["rev_id"] = id.RevisionID
	m["type"] = Revision
	return m
}

func (id *RevisionId) FromMap(m map[string]any) error {
	id.RevisionID = m["rev_id"].(string)
	if id.RevisionID == "" {
		return fmt.Errorf("%w: RevisionId required", ErrInvalidID)
	}

	if err := id.ItemId.FromMap(m); err != nil {
		return err
	}
	return nil
}

func (id *RevisionId) Validate() error {
	if id.RevisionID == "" {
		return fmt.Errorf("%w: RevisionId required", ErrInvalidID)
	}

	return id.ItemId.Validate()
}

func NewRevisionID(spaceID, envID, collID, itemID, id string) *ObjectId {
	return &ObjectId{Descriptor: &RevisionId{
		ItemId: ItemId{
			CollectionId: CollectionId{
				EnvironmentId: EnvironmentId{
					SpaceId: SpaceId{
						SpaceID: spaceID,
					},
					EnvironmentID: envID,
				},
				CollectionID: collID,
			},
			ItemID: itemID,
		},
		RevisionID: id,
	}}
}
